

import bpy
from bpy.props import BoolProperty, IntProperty, FloatProperty
from bpy.types import Operator, Panel, GizmoGroup
import time


################### SYNC LOGIC
####
###
##
#


#### VIEW SYNC PROPERTY GROUP
bpy.types.Scene.view_3d_index = bpy.props.IntProperty(name="View 3D ID", default=-1)

# **********************************************
class KT_SyncOptions(bpy.types.PropertyGroup):
    
    sync_view_distance: bpy.props.BoolProperty(name="Sync View Distance", default=True, description="Sync viewing distance between View Distance")    
    sync_view_location: bpy.props.BoolProperty(name="Sync View Location", default=True, description="Sync viewing location between 3D Views")    
    sync_view_rotation: bpy.props.BoolProperty(name="Sync View Rotation", default=True, description="Sync viewing rotation between 3D Views")    
    sync_camera_zoom: bpy.props.BoolProperty(name="Sync Camera Zoom", default=True, description="Sync camera zoom between 3D Views")    
    sync_camera_offset: bpy.props.BoolProperty(name="Sync Camera Offset", default=True, description="Sync camera offset between 3D Views")    
    sync_view_perspective: bpy.props.BoolProperty(name="Sync View Perspective", default=True, description="Sync viewing perspective between 3D Views")    
    sync_clip_start: bpy.props.BoolProperty(name="Sync Clip Start", default=True, description="Sync the start point of the viewing clip between 3D Views")    
    sync_clip_end: bpy.props.BoolProperty(name="Sync Clip End", default=True, description="Sync the end point of the viewing clip between 3D Views")    
    sync_focal_length: bpy.props.BoolProperty(name="Sync Focal Length", default=True, description="Sync focal length between 3D Views")   
    sync_view_distance_adjust: bpy.props.FloatProperty(name="Adjust View Distance:", default=1.0, min=0.01, max=5.0, description="Fine adjustment (Multiplier) for Sync View Distance. Lower = Closer | 1 = Default Distance")


# **********************************************    
class KT_VIEW3D_OT_sync_views(Operator):
    bl_idname = "view3d.sync_views"
    bl_label = "Sync Views"
    bl_description = "Synchronize selected views"
    

    def execute(self, context):
        sync_views(context)
        return {'FINISHED'}


#### PANEL
# **********************************************
class KT_VIEW3D_PT_SyncLock_MasterPanel(Panel):
    bl_idname = 'view3d.sync_lock_view_master'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "K-Tools"
    bl_label = "Sync | Lock Viewport"
    bl_options = {"DEFAULT_CLOSED"}
    
    def draw_header(self, context):
        layout = self.layout
        layout.label(text="", icon="UV_SYNC_SELECT")

    def draw(self, context):
        layout = self.layout
        scene = context.scene

# **********************************************
class KT_VIEW3D_PT_SyncPanel(Panel):
    bl_idname = 'view3d.view_sync_panel'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "View Sync"
    bl_label = "View Synchronization"
    bl_parent_id = 'view3d.sync_lock_view_master'
    #bl_options = {"DEFAULT_CLOSED"}
    
    def draw_header(self, context):
        layout = self.layout
        layout.label(text="", icon="UV_SYNC_SELECT")
        
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        active_screens = get_active_screens()
        view = context.space_data
        sync_options = scene.sync_options

        windows = bpy.context.window_manager.windows
        view3d_areas = [(window_index, area) for window_index, window in enumerate(windows) for area in window.screen.areas if area.type == 'VIEW_3D']

        box = layout.box()
        box.label(text='View 3D Sync:')

        row = box.row(align=True)
        row.operator("view3d.sync_views", text='Single Match')
        row.prop(scene, "real_time_sync", toggle=True)   

        row = box.row(align=True)
        row.prop(scene, "master_view_index", text="Master ID:")
        row.prop(scene, "sync_refresh_rate", text='Refresh Rate:') 
        row = box.row(align=True)
        row.prop(sync_options, "sync_view_distance_adjust", text="Adjust View Distance")
        

        # Clip Start & End
        row = box.row()
        row.label(text='Clip Start:')
        row.label(text='Clip End:')
        row.label(text='Focal Length:')
        grid = box.grid_flow(row_major=True, columns=3, align=True)        
        grid.prop(view, "clip_start", text="")
        grid.prop(view, "clip_end", text="")
        row = box.row(align=True)
        row.operator("view3d.local_view_all_areas", text='Local View')
        row.operator("view3d.local_view_all_areas_with_lights", text='Local View + Lights')
         
        # Focal Length
        grid.prop(view, "lens", text="")

        box.label(text="Available 3D Views:")

        grid = box.grid_flow(columns=3, even_columns=True, even_rows=False)

        current_window_index = -1
        for i, (window_index, area) in enumerate(view3d_areas):
            if window_index != current_window_index:

                current_window_index = window_index
                col = grid.column()
                col.label(text=f"Window {current_window_index}")

            row = col.row(align=True)
            if hasattr(scene, f"sync_view_{i}"):
                row.prop(scene, f"sync_view_{i}", text=f"View {i}")              

        row = box.row(align=True)                
        row.operator("view3d.show_current_view_id", text='Identify 3D View')
        row.operator("view3d.update_view_count")
        row = box.row()
        row.label(text=f"Last ID Checked: {context.scene.view_3d_index}")
        row.label(text=f"Current Master: {scene.master_view_index}")
                

    def get_window_and_view_index(self, area):
        """Função auxiliar para retornar o índice da janela e da view correspondente."""
        windows = bpy.context.window_manager.windows
        global_view_index = -1
        for window_index, window in enumerate(windows):
            for view_area in window.screen.areas:
                if view_area.type == 'VIEW_3D':
                    global_view_index += 1
                    if view_area == area:
                        return window_index, global_view_index
        return -1, -1



### SUBPANEL - SYNC OPTIONS 
# **********************************************
class KT_VIEW3D_PT_options_buttons_panel(bpy.types.Panel):
    bl_label = "Sync Options"
    bl_idname = "KT_VIEW3D_PT_options_buttons_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'K-Tools'
    bl_parent_id = "view3d.view_sync_panel"
    bl_options = {"DEFAULT_CLOSED"}
    
    def draw_header(self, context):
        layout = self.layout
        layout.label(text="", icon="OPTIONS")

    def draw(self, context):
        layout = self.layout
        scene = context.scene
        sync_options = scene.sync_options  # Acesso ao PropertyGroup

        box = layout.box()
        grid = box.grid_flow(row_major=False, columns=2, align=True)         
        box.label(text='View Sync Options:') 
        grid = box.grid_flow(row_major=False, columns=2, align=True)      
       
        # Adiciona os botões booleanos no painel
        grid.prop(sync_options, "sync_view_distance", text='Distance')
        grid.prop(sync_options, "sync_view_location", text='Location')
        grid.prop(sync_options, "sync_view_rotation", text='Rotation')
        grid.prop(sync_options, "sync_camera_zoom", text='Camera Zoom')
        grid.prop(sync_options, "sync_camera_offset", text='Camera Offset')
        grid.prop(sync_options, "sync_view_perspective", text='Perspective')
        grid.prop(sync_options, "sync_clip_start", text='Clip Start')
        grid.prop(sync_options, "sync_clip_end", text='Clip End')
        grid.prop(sync_options, "sync_focal_length", text='Focal Lenght')
        grid.prop(sync_options, "dont_exclude_lights", text='Dont Hide Lights')

#### MAIN OPERATOR
def sync_views(context):
    scene = context.scene
    sync_options = scene.sync_options  # Acesso ao PropertyGroup
    view3d_areas = [(window_index, area) for window_index, window in enumerate(bpy.context.window_manager.windows) for area in window.screen.areas if area.type == 'VIEW_3D']

    master_index = scene.master_view_index
    if master_index >= len(view3d_areas):
        return

    _, master_area = view3d_areas[master_index]
    master_space = master_area.spaces.active

    for i, (_, area) in enumerate(view3d_areas):
        if i != master_index and getattr(scene, f"sync_view_{i}", False):
            space = area.spaces.active

            # Sincroniza apenas as propriedades que estão habilitadas
            if sync_options.sync_view_distance:
                space.region_3d.view_distance = master_space.region_3d.view_distance * sync_options.sync_view_distance_adjust
            
            if sync_options.sync_view_location:
                space.region_3d.view_location = master_space.region_3d.view_location
            
            if sync_options.sync_view_rotation:
                space.region_3d.view_rotation = master_space.region_3d.view_rotation
            
            if sync_options.sync_camera_zoom:
                space.region_3d.view_camera_zoom = master_space.region_3d.view_camera_zoom
            
            if sync_options.sync_camera_offset:
                space.region_3d.view_camera_offset = master_space.region_3d.view_camera_offset
            
            if sync_options.sync_view_perspective:
                space.region_3d.view_perspective = master_space.region_3d.view_perspective
            
            if sync_options.sync_clip_start:
                space.clip_start = master_space.clip_start
            
            if sync_options.sync_clip_end:
                space.clip_end = master_space.clip_end
            
            if sync_options.sync_focal_length:
                space.lens = master_space.lens

####

#### UPDATE VIEW COUNT

class KT_VIEW3D_OT_update_view_count(Operator):
    bl_idname = "view3d.update_view_count"
    bl_label = "Update View List"
    bl_description = "Update the count of View3D areas and refresh boolean buttons"

    def execute(self, context):
        
        update_view_properties(self, context)
    
        # Força uma atualização da interface
        for area in context.screen.areas:
            area.tag_redraw()
        return {'FINISHED'}

def get_active_screens():
    return [window.screen for window in bpy.context.window_manager.windows]

def update_view_properties(self, context):
    view3d_areas = [area for window in bpy.context.window_manager.windows for area in window.screen.areas if area.type == 'VIEW_3D']
    view3d_count = len(view3d_areas)


    # Contador global de views para numeração sequencial
    global_view_index = 0
    
    for i in range(view3d_count, 20):  # Assume um limite razoável
        if hasattr(bpy.types.Scene, f"sync_view_{i}"):
            delattr(bpy.types.Scene, f"sync_view_{i}")
    
    for i in range(view3d_count):
        if not hasattr(bpy.types.Scene, f"sync_view_{i}"):
            setattr(bpy.types.Scene, f"sync_view_{i}", BoolProperty(
                name=f"Sync View {i}",
                description=f"Synchronize View3D {i}",
                default=False
            ))

    bpy.types.Scene.master_view_index = IntProperty(name="Master View Index", description="Index of the master View3D", default=0, min=0,  max=max(0, view3d_count - 1))


last_interaction_time = 0
mouse_button_pressed = False  # track mouse button state

# **********************************************
def real_time_sync_timer():
    global last_interaction_time, mouse_button_pressed
    scene = bpy.context.scene
    
    if scene.real_time_sync and not mouse_button_pressed:  # Only sync if no button is in True
        current_time = time.time()
        if current_time - last_interaction_time > scene.sync_pause_duration:
            sync_views(bpy.context)
        return scene.sync_refresh_rate
    return None

# **********************************************
class KT_WM_OT_show_message(bpy.types.Operator):
    bl_label = "Show Message Box"
    bl_idname = "wm.show_message"

    message: bpy.props.StringProperty(default="View ID:")
    title: bpy.props.StringProperty(default="Current View Index")
    icon: bpy.props.StringProperty(default='INFO')

    def execute(self, context):
        ShowMessageBox(self.message, self.title, self.icon)
        return {'FINISHED'}

# **********************************************
def ShowMessageBox(message="", title="Message Box", icon='INFO'):
    def draw(self, context):
        self.layout.label(text=message)
    bpy.context.window_manager.popup_menu(draw, title=title, icon=icon)

####


# **********************************************
class KT_VIEW3D_OT_ShowViewID(bpy.types.Operator):
    """Exibe o índice global da janela e da View 3D onde o botão foi clicado"""
    bl_idname = "view3d.show_current_view_id"
    bl_label = "Show Current Window and Global View 3D ID"

    def execute(self, context):
        # Obter todas as janelas abertas
        windows = bpy.context.window_manager.windows
        current_window_index = -1
        current_view_index = -1  # Índice global de visualizações
        global_view_index = -1   # Índice contínuo global de views

        for window_index, window in enumerate(windows):
            for area in window.screen.areas:
                if area.type == 'VIEW_3D':
                    global_view_index += 1  # Aumentar o índice global
                    if area == context.area:
                        current_window_index = window_index
                        current_view_index = global_view_index
                        break
            if current_window_index != -1:
                break

        if current_window_index != -1 and current_view_index != -1:
            # Definir os índices no contexto da cena
            context.scene.view_3d_index = current_view_index
            message = f"Window {current_window_index}, View {current_view_index}"
        else:
            message = "Nenhuma View 3D ativa."

        # Mostrar a mensagem no console e em uma caixa de diálogo
        self.report({'INFO'}, message)
        ShowMessageBox(message, title="Current View 3D ID", icon='INFO')

        return {'FINISHED'}
####


#### SYNC LOCAL VIEW MODE - MANUAL BUTTON

# **********************************************
class KT_LOCAL_VIEW_OT_all_areas(bpy.types.Operator):
    bl_idname = "view3d.local_view_all_areas"
    bl_label = "Local View em Todas as Vistas"
    bl_description = "Set Local View for all 3D Views"

    def execute(self, context):
        selected_objects = context.selected_objects
        active_object = context.active_object

        # Itera sobre todas as janelas e todas as áreas do tipo VIEW_3D
        for window in bpy.context.window_manager.windows:
            for area in window.screen.areas:
                if area.type == 'VIEW_3D':
                    # Cria um novo contexto para cada área
                    override = context.copy()
                    override['window'] = window
                    override['screen'] = window.screen
                    override['area'] = area
                    override['region'] = area.regions[-1]  # Região 3D
                    
                    with context.temp_override(**override):
                        bpy.ops.view3d.localview()

        bpy.ops.object.select_all(action='DESELECT')
        for obj in selected_objects:
            obj.select_set(True)
        if active_object:
            context.view_layer.objects.active = active_object

        return {'FINISHED'}

# **********************************************
class KT_LOCAL_VIEW_OT_all_areas_with_lights(bpy.types.Operator):
    bl_idname = "view3d.local_view_all_areas_with_lights"
    bl_label = "Set Local View for all 3D Views while keeping visible lights"
    bl_description = "Entra/Sai do Local View com objetos selecionados e mantém luzes visíveis em todas as vistas 3D"


# **********************************************
class KT_LOCAL_VIEW_OT_all_areas_with_lights(bpy.types.Operator):
    bl_idname = "view3d.local_view_all_areas_with_lights"
    bl_label = "Local View em Todas as Vistas (Mantendo Luzes Visíveis)"
    bl_description = "Entra/Sai do Local View com objetos selecionados e mantém luzes visíveis em todas as vistas 3D"

    def execute(self, context):
        selected_objects = context.selected_objects
        active_object = context.active_object

        # Selects all active lights that are in the View Layer
        lights = [obj for obj in context.scene.objects if obj.type == 'LIGHT' and obj.visible_get()]

        # Adds lights to selection
        for light in lights:
            light.select_set(True)

        for window in bpy.context.window_manager.windows:
            for area in window.screen.areas:
                if area.type == 'VIEW_3D':

                    override = context.copy()
                    override['window'] = window
                    override['screen'] = window.screen
                    override['area'] = area
                    override['region'] = area.regions[-1]
                    
                    # Execute the Local View command
                    with context.temp_override(**override):
                        bpy.ops.view3d.localview()

        # Deselect lights before framing selection
        bpy.ops.object.select_all(action='DESELECT')
        for obj in selected_objects:
            obj.select_set(True)

        # Frame the selection (Frame Selected) in each VIEW_3D area
        for window in bpy.context.window_manager.windows:
            for area in window.screen.areas:
                if area.type == 'VIEW_3D':
                    # Cria um novo contexto para cada área
                    override = context.copy()
                    override['window'] = window
                    override['screen'] = window.screen
                    override['area'] = area
                    override['region'] = area.regions[-1]  # Região 3D
                    
                    # Executa o comando de Frame Selected
                    with context.temp_override(**override):
                        bpy.ops.view3d.view_selected()

        # Restaura a seleção original
        for obj in selected_objects:
            obj.select_set(True)
        if active_object:
            context.view_layer.objects.active = active_object

        return {'FINISHED'}




################ LOCK LOGIC
####
###
##
#



# **********************************************
class KT_LockRotationProps(bpy.types.PropertyGroup):
    lock_rotation: bpy.props.BoolProperty(name="Lock Rotation", default=False, update=lambda self, context: update_lock_rotation(self, context))


# **********************************************
class KT_ViewLockItem(bpy.types.PropertyGroup):
        window_index: bpy.props.IntProperty()
        local_view_index: bpy.props.IntProperty()
        global_view_index: bpy.props.IntProperty()
        lock_rotation: bpy.props.BoolProperty(name="Lock Rotation", default=False, update=lambda self, context: update_lock_rotation(self, context))


# **********************************************
def set_lock_rotation(area, value):
    if area.type == 'VIEW_3D':
        region_3d = area.spaces.active.region_3d
        region_3d.lock_rotation = value


# **********************************************
def update_lock_rotation(self, context):
    wm = context.window_manager
    window = wm.windows[self.window_index]
    area = [a for a in window.screen.areas if a.type == 'VIEW_3D'][self.local_view_index]
    set_lock_rotation(area, self.lock_rotation)


# **********************************************
class KT_VIEW3D_OT_force_update(bpy.types.Operator):
    bl_idname = "view3d.force_lock_update"
    bl_label = "Force Update"
    bl_description = "Force update of the 3D Views list"

    def execute(self, context):
        wm = context.window_manager
        wm.view_sync_data.clear()
        
        global_view_index = 0

        for window_idx, window in enumerate(wm.windows):
            for area_idx, area in enumerate([a for a in window.screen.areas if a.type == 'VIEW_3D']):
                item = wm.view_sync_data.add()
                item.window_index = window_idx
                item.local_view_index = area_idx  # Local index for window
                item.global_view_index = global_view_index  # Global sequential numbering
                item.lock_rotation = area.spaces.active.region_3d.lock_rotation
                global_view_index += 1  # Increases the global index
        
        return {'FINISHED'}


# **********************************************
# UPTADE THE LIST WHEN THE CONTEXT IS SWITCHED
@bpy.app.handlers.persistent
def update_view_sync_data(scene):
    wm = bpy.context.window_manager
    wm.view_sync_data.clear()


    global_view_index = 0

    for window_idx, window in enumerate(wm.windows):
        for area_idx, area in enumerate([a for a in window.screen.areas if a.type == 'VIEW_3D']):
            item = wm.view_sync_data.add()
            item.window_index = window_idx
            item.local_view_index = area_idx  # Índice local
            item.global_view_index = global_view_index  # Índice global sequencial
            item.lock_rotation = area.spaces.active.region_3d.lock_rotation
            global_view_index += 1  # Incrementa o índice global



# **********************************************
class KT_CameraViewToggleGizmo(GizmoGroup):
    bl_idname = "VIEW3D_GGT_camera_view_toggle"
    bl_label = "Camera View Toggle Button"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'WINDOW'
    bl_options = {'PERSISTENT', 'SCALE'}


    @classmethod
    def poll(cls, context):
        if context.area.type == 'VIEW_3D':
            region_3d = context.space_data.region_3d
            if region_3d.lock_rotation and context.window_manager.show_gizmo_special:
                if not context.space_data.show_gizmo:
                    context.space_data.show_gizmo = True 
                return True
        return False


    def draw_prepare(self, context):
        if self.move_gizmo and self.zoom_gizmo:
            region = context.region

            self.solid_gizmo.matrix_basis[0][3] = region.width / 10 - 35 
            self.solid_gizmo.matrix_basis[1][3] = 30

            self.material_gizmo.matrix_basis[0][3] = region.width / 10
            self.material_gizmo.matrix_basis[1][3] = 30

            self.render_gizmo.matrix_basis[0][3] = region.width / 10 + 35
            self.render_gizmo.matrix_basis[1][3] = 30

            self.shading_gizmo.matrix_basis[0][3] = region.width / 10 + 70
            self.shading_gizmo.matrix_basis[1][3] = 30

            self.camera_gizmo.matrix_basis[0][3] = region.width / 2 - 60
            self.camera_gizmo.matrix_basis[1][3] = 30

            self.move_gizmo.matrix_basis[0][3] = region.width / 2 - 20
            self.move_gizmo.matrix_basis[1][3] = 30

            self.zoom_gizmo.matrix_basis[0][3] = region.width / 2 + 20
            self.zoom_gizmo.matrix_basis[1][3] = 30


    def setup(self, context):

       # Set Solid Mode
        self.solid_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.solid_gizmo.icon = 'SHADING_SOLID'
        self.solid_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.solid_gizmo.color = 0.078, 0.15, 0.23
        self.solid_gizmo.alpha = 0.8
        self.solid_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.solid_gizmo.alpha_highlight = 0.5
        self.solid_gizmo.scale_basis = 15
        self.solid_gizmo.target_set_operator("view3d.toggle_solid_shading_view")

        # Set Material Mode
        self.material_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.material_gizmo.icon = 'MATERIAL'
        self.material_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.material_gizmo.color = 0.078, 0.15, 0.23
        self.material_gizmo.alpha = 0.8
        self.material_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.material_gizmo.alpha_highlight = 0.5
        self.material_gizmo.scale_basis = 15
        self.material_gizmo.target_set_operator("view3d.toggle_material_shading_view") 

        # Set Render Mode
        self.render_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.render_gizmo.icon = 'SHADING_RENDERED'
        self.render_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.render_gizmo.color = 0.078, 0.15, 0.23
        self.render_gizmo.alpha = 0.8
        self.render_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.render_gizmo.alpha_highlight = 0.5
        self.render_gizmo.scale_basis = 15
        self.render_gizmo.target_set_operator("view3d.toggle_render_shading_view") 

        # Shading Panel
        self.shading_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.shading_gizmo.icon = 'MENU_PANEL'
        self.shading_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.shading_gizmo.color = 0.078, 0.15, 0.23
        self.shading_gizmo.alpha = 0.8
        self.shading_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.shading_gizmo.alpha_highlight = 0.5
        self.shading_gizmo.scale_basis = 15
        self.shading_gizmo.target_set_operator("view3d.call_shading_popover")

        # Set Camera View
        self.camera_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.camera_gizmo.icon = 'CAMERA_DATA'
        self.camera_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.camera_gizmo.color = 0.078, 0.15, 0.23
        self.camera_gizmo.alpha = 0.8
        self.camera_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.camera_gizmo.alpha_highlight = 0.5
        self.camera_gizmo.scale_basis = 15
        self.camera_gizmo.target_set_operator("view3d.kt_toggle_camera_view") 

        # View Pan
        self.move_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.move_gizmo.icon = 'VIEW_PAN'
        self.move_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.move_gizmo.color = 0.078, 0.15, 0.23
        self.move_gizmo.alpha = 0.8
        self.move_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.move_gizmo.alpha_highlight = 0.5
        self.move_gizmo.scale_basis = 15
        self.move_gizmo.target_set_operator("view3d.move")

        # Zoom
        self.zoom_gizmo = self.gizmos.new("GIZMO_GT_button_2d")
        self.zoom_gizmo.icon = 'ZOOM_IN'
        self.zoom_gizmo.draw_options = {'BACKDROP', 'OUTLINE'}
        self.zoom_gizmo.color = 0.078, 0.15, 0.23
        self.zoom_gizmo.alpha = 0.8
        self.zoom_gizmo.color_highlight = 0.3, 0.8, 1.0
        self.zoom_gizmo.alpha_highlight = 0.5
        self.zoom_gizmo.scale_basis = 15
        self.zoom_gizmo.target_set_operator("view3d.zoom")

 


# Function to disable or enable gizmos and overlays based on the boolean value
# **********************************************
def update_disable_gizmos(self, context):
    wm = context.window_manager
    for window in wm.windows:
        for area in window.screen.areas:
            if area.type == 'VIEW_3D':
                region_3d = area.spaces.active.region_3d
                # Check if lock_rotation is active
                if region_3d.lock_rotation:
                    # Disable or enable gizmos based on the value of 'disable_gizmos'
                    area.spaces.active.show_gizmo_navigate = not self.disable_gizmos
                    area.spaces.active.show_gizmo_tool = not self.disable_gizmos
                    area.spaces.active.show_gizmo_context = not self.disable_gizmos
                    area.spaces.active.show_gizmo_empty_image = not self.disable_gizmos
                    area.spaces.active.show_gizmo_empty_force_field = not self.disable_gizmos
                    area.spaces.active.show_gizmo_light_size = not self.disable_gizmos
                    area.spaces.active.show_gizmo_light_look_at = not self.disable_gizmos
                    area.spaces.active.show_gizmo_camera_lens = not self.disable_gizmos
                    area.spaces.active.show_gizmo_camera_dof_distance = not self.disable_gizmos

                    area.spaces.active.overlay.show_overlays = not self.disable_gizmos

                    area.spaces.active.show_region_header = not self.disable_gizmos



# **********************************************
class KT_VIEW3D_OT_toggle_gizmos(bpy.types.Operator):
    '''Disable/Enable some predefined items from the viewport that have Lock Rotation enabled'''
    bl_idname = "view3d.toggle_gizmos"
    bl_label = "Toggle Gizmos and Overlays"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        wm = context.window_manager
        gizmos_active = False
        
        for window in wm.windows:
            for area in window.screen.areas:
  
                if area.type == 'VIEW_3D':
                    region_3d = area.spaces.active.region_3d

                    if region_3d.lock_rotation:

                        gizmos_active = area.spaces.active.show_gizmo_navigate
                        
                        area.spaces.active.show_gizmo_navigate = not gizmos_active
                        area.spaces.active.show_gizmo_tool = not gizmos_active
                        area.spaces.active.show_gizmo_context = not gizmos_active
                        area.spaces.active.show_gizmo_empty_image = not gizmos_active
                        area.spaces.active.show_gizmo_empty_force_field = not gizmos_active
                        area.spaces.active.show_gizmo_light_size = not gizmos_active
                        area.spaces.active.show_gizmo_light_look_at = not gizmos_active
                        area.spaces.active.show_gizmo_camera_lens = not gizmos_active
                        area.spaces.active.show_gizmo_camera_dof_distance = not gizmos_active

                        area.spaces.active.overlay.show_overlays = not gizmos_active
                        
                        area.spaces.active.show_region_header = not gizmos_active
        
        return {'FINISHED'}




### GIZMO CALL OPERATORS
# **********************************************
class KT_VIEW3D_OT_toggle_camera_view(Operator):
    """Switch between camera view and perspective view"""
    bl_idname = "view3d.kt_toggle_camera_view"
    bl_label = "Toggle Camera View"
    
    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':
            space = area.spaces.active
            if space.type == 'VIEW_3D':
                space.region_3d.view_perspective = 'CAMERA' if space.region_3d.view_perspective != 'CAMERA' else 'PERSP'

        return {'FINISHED'}



# **********************************************
class KT_VIEW3D_OT_toggle_solid_shading_view(Operator):
    """Toggle to Solid Shading."""
    bl_idname = "view3d.toggle_solid_shading_view"
    bl_label = "Toggle Solid Shading"

    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':
            space = area.spaces.active
            if space.type == 'VIEW_3D':
                # Switches between Solid and Material
                shading = space.shading
                if shading.type == 'SOLID':
                    shading.type = 'SOLID'  # Switch to Solid if in another mode
                else:
                    shading.type = 'SOLID'  # Switch to Solid if in another mode
        return {'FINISHED'}



# **********************************************
class KT_VIEW3D_OT_toggle_material_shading_view(Operator):
    """Toggles between Material and Solid Shading."""
    bl_idname = "view3d.toggle_material_shading_view"
    bl_label = "Toggle Material Shading"

    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':
            space = area.spaces.active
            if space.type == 'VIEW_3D':
                # Switches between Material and Solid
                shading = space.shading
                if shading.type == 'MATERIAL':
                    shading.type = 'SOLID'  # Switch to Solid if in Material
                else:
                    shading.type = 'MATERIAL'  # Switch to Material if in another mode
        return {'FINISHED'}



# **********************************************  
class KT_VIEW3D_OT_toggle_render_shading_view(Operator):
    """Toggles between Rendered and Solid Shading."""
    bl_idname = "view3d.toggle_render_shading_view"
    bl_label = "Toggle Render Shading"

    def execute(self, context):
        area = context.area
        if area.type == 'VIEW_3D':
            space = area.spaces.active
            if space.type == 'VIEW_3D':
                # Toggles between Rendered and previous mode
                shading = space.shading
                if shading.type == 'RENDERED':
                    shading.type = 'SOLID'  # Switch to Solid if in Rendered
                else:
                    shading.type = 'RENDERED'  # Switch to Rendered if in another mode
        return {'FINISHED'}



# **********************************************
class KT_VIEW3D_OT_call_shading_popover(bpy.types.Operator):
    ''' Shading Menu '''
    bl_idname = "view3d.call_shading_popover"
    bl_label = "Call Shading Popover"
    
    _timer = None
    _duration = 0.1  # Duration in seconds before closing the popover

    def modal(self, context, event):
        # If the mouse moves, check the position
        if event.type == 'MOUSEMOVE':
            pass

        # Finish the operator when the ESC key is pressed
        if event.type in {'ESC'}:
            return self.finish(context)

        # If the duration time has passed, finish the operator
        if self._timer is not None and event.type == 'TIMER':
            return self.finish(context)

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        # Call the popover panel
        bpy.ops.wm.call_panel(name="VIEW3D_PT_shading")

        # Add a modal handler
        context.window_manager.modal_handler_add(self)

        # Start a timer to close the popover after _duration seconds
        self._timer = context.window_manager.event_timer_add(self._duration, window=context.window)

        return {'RUNNING_MODAL'}

    def finish(self, context):
        # Remove the timer, if it exists
        if self._timer:
            context.window_manager.event_timer_remove(self._timer)
            self._timer = None

        # Redraw the area
        context.area.tag_redraw()
        return {'CANCELLED'}



 
### LOCK PANEL

bpy.types.WindowManager.show_gizmo_special = bpy.props.BoolProperty(name="Show Gizmo While Lock", default=True, description='Disable/Enable some predefined items from the viewport that have Lock Rotation enabled')


# **********************************************
class KT_VIEW3D_PT_lock_rotation_panel(bpy.types.Panel):
    bl_label = "Lock Rotation"
    bl_idname = "KT_VIEW3D_PT_lock_rotation_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    #bl_category = "K-Tools"
    bl_parent_id = 'view3d.sync_lock_view_master'
    bl_options = {"DEFAULT_CLOSED"}

    def draw_header(self, context):
        layout = self.layout
        layout.label(text="", icon="LOCKED")

    def draw(self, context):
        layout = self.layout
        wm = context.window_manager

        layout = self.layout
        scene = context.scene
        active_screens = get_active_screens()
        view = context.space_data
        sync_options = scene.sync_options

        box = layout.box()
        box.label(text="Available 3D Views:")

        # Organiza as views por Window
        window_views = {}
        for item in wm.view_sync_data:
            window_index = item.window_index
            if window_index not in window_views:
                window_views[window_index] = []
            window_views[window_index].append(item)

        grid = box.grid_flow(columns=len(window_views), even_columns=True, even_rows=False)
        for window_index, views in window_views.items():
            col = grid.column()
            #col.label(text=f"Window {window_index}", icon='WINDOW')
            col.label(text=f"Window {window_index}:")
            for item in views:
                col.prop(item, "lock_rotation", text=f"Lock View {item.global_view_index}")

        row = box.row(align=True)                
        row.operator("view3d.show_current_view_id", text='Identify 3D View')
        row.operator("view3d.force_lock_update", text="Update Lock List")
        row = box.row()
        row.label(text=f"Last ID Checked: {context.scene.view_3d_index}")

        row = box.row(align=True)
        row.prop(wm, "show_gizmo_special", text="Special Gizmos", toggle=True)
        #row.prop(wm, "disable_gizmos", text="Set Clean View", toggle=True)
        row.operator("view3d.toggle_gizmos", text="Set Clean View")

    

classes = (
### SYNC

            KT_VIEW3D_PT_SyncLock_MasterPanel,
            KT_VIEW3D_PT_SyncPanel, 
            KT_VIEW3D_PT_options_buttons_panel,
            KT_VIEW3D_OT_sync_views,
            KT_VIEW3D_OT_update_view_count,
            KT_VIEW3D_OT_ShowViewID,
            KT_WM_OT_show_message,
            KT_LOCAL_VIEW_OT_all_areas,
            KT_LOCAL_VIEW_OT_all_areas_with_lights,
            KT_SyncOptions,
            
### LOCK
            KT_VIEW3D_PT_lock_rotation_panel,
            KT_CameraViewToggleGizmo,
            KT_LockRotationProps,
            KT_ViewLockItem,
            KT_VIEW3D_OT_toggle_camera_view,
            KT_VIEW3D_OT_toggle_solid_shading_view,
            KT_VIEW3D_OT_toggle_material_shading_view,
            KT_VIEW3D_OT_toggle_render_shading_view,
            KT_VIEW3D_OT_force_update,
            KT_VIEW3D_OT_call_shading_popover,
            KT_VIEW3D_OT_toggle_gizmos,
)


def register():
    for cls in classes:
        bpy.utils.register_class(cls)
        
    bpy.types.Scene.sync_options = bpy.props.PointerProperty(type=KT_SyncOptions)
    bpy.types.Scene.master_view_index = IntProperty(name="Master View Index",  description="Index of the master View3D", default=0, min=0, update=update_view_properties)
    bpy.types.Scene.real_time_sync = BoolProperty( name="Real-time Sync", description="Enable real-time synchronization", default=False, update=lambda self, context: bpy.app.timers.register(real_time_sync_timer) if self.real_time_sync else bpy.app.timers.unregister(real_time_sync_timer))
    bpy.types.Scene.sync_refresh_rate = FloatProperty(name="Refresh Rate", description="Refresh rate for real-time synchronization (in seconds)", default=0.1, min=0.01, max=1.0)
    bpy.types.Scene.sync_pause_duration = FloatProperty( name="Interaction Pause", description="Duration to pause sync after user interaction (in seconds)", default=0.5, min=0.1, max=2.0)

    if not hasattr(bpy.types.Scene, "master_view_index"): bpy.types.Scene.master_view_index = IntProperty(name="Master View Index", description="Index of the master View3D", default=0,  min=0, max=0)  # Será atualizado pela função update_view_properties
              
    bpy.app.handlers.load_post.append(update_view_properties)
    
    update_view_properties(None, bpy.context)

### LOCK VIEW

    bpy.types.WindowManager.view_sync_data = bpy.props.CollectionProperty(type=KT_ViewLockItem)
    bpy.types.WindowManager.disable_gizmos = bpy.props.BoolProperty(name="Disable Gizmos", default=False, update=update_disable_gizmos)


def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.Scene.sync_options
    
    del bpy.types.Scene.master_view_index
    del bpy.types.Scene.real_time_sync
    del bpy.types.Scene.sync_refresh_rate
    del bpy.types.Scene.sync_pause_duration

    if real_time_sync_timer in bpy.app.timers.get_all():
        bpy.app.timers.unregister(real_time_sync_timer)

    if hasattr(bpy.types.Scene, "master_view_index"):
        del bpy.types.Scene.master_view_index

    for attr in dir(bpy.types.Scene):
        if attr.startswith("sync_view_"):
            delattr(bpy.types.Scene, attr)

#### LOCK VIEW

    del bpy.types.WindowManager.view_sync_data    
    del bpy.types.WindowManager.disable_gizmos

if __name__ == "__main__":
    register()
